{% extends "!layout.html" %}
{% block footer %}
	{{ super() }}
	
    <script type="text/javascript">

      var _gaq = _gaq || [];
      _gaq.push(['_setAccount', 'UA-155626-14']);
      _gaq.push(['_trackPageview']);

      (function() {
        var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
        ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
        var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
      })();

    </script>

{% endblock %}
{% block extrahead %}
	{{ super() }}


   <style type="text/css">
        span.brand { color: #a00;}
   </style> 
   
{% endblock %}
{% block copyright %}
{% endblock %}
